@@ -8,7 +8,7 @@ Add fltk-webview to your fltk application's Cargo.toml file:
 ```toml
 [dependencies]
 fltk = "1"
-fltk-webview = "0.1"
+fltk-webview = "0.2"
 ```
 
 Then you can embed a webview using fltk_webview::Webview::create:
@@ -29,7 +29,7 @@ fn main() {
 
     let mut wv = fltk_webview::Webview::create(false, &mut wv_win);
     wv.navigate("https://google.com");
-    
+
     // the webview handles the main loop
     wv.run();
 }
@@ -55,8 +55,6 @@ use std::{
 };
 use webview_official_sys as wv;
 
-static mut RUNNING: bool = true;
-
 pub(crate) trait FlString {
     fn safe_new(s: &str) -> CString;
 }
@@ -117,9 +115,10 @@ impl Webview {
                     &mut win.raw_handle() as *mut *mut raw::c_void as *mut raw::c_void,
                 );
                 win.draw(move |w| wv::webview_set_size(inner, w.w(), w.h(), 0));
-                win.parent().unwrap().set_callback(|_| {
+                let mut topwin =
+                    window::Window::from_widget_ptr(win.top_window().unwrap().as_widget_ptr());
+                topwin.set_callback(|_| {
                     if app::event() == enums::Event::Close {
-                        RUNNING = false;
                         std::process::exit(0);
                     }
                 });
@@ -129,16 +128,15 @@ impl Webview {
                 pub enum NSWindow {}
                 extern "C" {
                     pub fn make_delegate(child: *mut NSWindow, parent: *mut NSWindow);
-                    pub fn send_event(event: *mut raw::c_void, data: *mut raw::c_void) -> i32;
                 }
                 let handle = win.raw_handle();
                 inner = wv::webview_create(debug as i32, handle as _);
                 make_delegate(wv::webview_get_window(inner) as _, handle as _);
-                app::add_system_handler(Some(send_event), wv::webview_get_window(inner) as _);
                 win.draw(move |w| wv::webview_set_size(inner, w.w(), w.h(), 0));
-                win.parent().unwrap().set_callback(|_| {
+                let mut topwin =
+                    window::Window::from_widget_ptr(win.top_window().unwrap().as_widget_ptr());
+                topwin.set_callback(|_| {
                     if app::event() == enums::Event::Close {
-                        RUNNING = false;
                         std::process::exit(0);
                     }
                 });
@@ -154,7 +152,10 @@ impl Webview {
                     pub fn my_get_win(wid: *mut GtkWindow) -> *mut GdkWindow;
                     pub fn my_get_xid(w: *mut GdkWindow) -> u64;
                     pub fn x_init(disp: *mut Display, child: u64, parent: u64);
-                    pub fn my_xembed(disp: *mut Display, child: u64, parent: u64);
+                    pub fn g_idle_add(
+                        cb: Option<extern "C" fn(*mut raw::c_void) -> bool>,
+                        data: *mut raw::c_void,
+                    );
                 }
                 gtk_init(&mut 0, std::ptr::null_mut());
                 inner = wv::webview_create(debug as i32, std::ptr::null_mut() as _);
@@ -166,18 +167,24 @@ impl Webview {
                 let flxid = win.raw_handle();
                 if win_manager("gnome-session") {
                     win.draw(move |w| {
+                        x_init(app::display() as _, xid, flxid);
+                        app::sleep(0.03);
                         wv::webview_set_size(inner, w.w(), w.h(), 0);
-                        my_xembed(app::display() as _, xid, flxid);
                     });
                     win.flush();
                 } else {
                     x_init(app::display() as _, xid, flxid);
                     win.draw(move |w| wv::webview_set_size(inner, w.w(), w.h(), 0));
-
                 }
-                win.parent().unwrap().set_callback(|_| {
+                extern "C" fn cb(_data: *mut raw::c_void) -> bool {
+                    app::check()
+                }
+                g_idle_add(Some(cb), std::ptr::null_mut());
+                let mut topwin =
+                    window::Window::from_widget_ptr(win.top_window().unwrap().as_widget_ptr());
+                topwin.set_callback(move |_| {
                     if app::event() == enums::Event::Close {
-                        RUNNING = false;
+                        std::process::exit(0);
                     }
                 });
             }
@@ -273,19 +280,12 @@ impl Webview {
 
     /// Run the main loop of the webview
     pub fn run(&self) {
-        unsafe {
-            #[cfg(not(any(target_os = "macos", target_os = "windows")))]
-            {
-                extern "C" {
-                    pub fn gtk_main_iteration_do(val: bool) -> bool;
-                }
-                while gtk_main_iteration_do(true) && RUNNING {
-                    app::check();
-                }
-            }
-            #[cfg(any(target_os = "macos", target_os = "windows"))]
-            wv::webview_run(*self.inner) 
+        #[cfg(any(target_os = "macos", target_os = "windows"))] 
+        {
+            app::run().unwrap();
         }
+        #[cfg(not(any(target_os = "macos", target_os = "windows")))] 
+        unsafe { wv::webview_run(*self.inner) }
     }
 
     /// Set the size of the webview window
@@ -300,7 +300,10 @@ fn win_manager(prog: &str) -> bool {
     if let Ok(sm) = sm {
         let pid = sm.split("/").last();
         if let Some(pid) = pid {
-            match std::process::Command::new("ps").args(&["-p", pid, "-o", "comm="]).output() {
+            match std::process::Command::new("ps")
+                .args(&["-p", pid, "-o", "comm="])
+                .output()
+            {
                 Ok(out) => {
                     if String::from_utf8_lossy(&out.stdout).contains(prog) {
                         true
